using System.Diagnostics;
using System.Globalization;
using System.Reflection;
using System.Text.Json;
using System.Text.Json.Serialization;

namespace Tracker
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            List<GeoPoint> points = new List<GeoPoint>();
            string currentDir = $@"{Application.StartupPath}\..\..\..\tracker_files";
            string[] files = Directory.GetFiles(currentDir, "*.json", SearchOption.TopDirectoryOnly);
            foreach (string filename in files)
            {
                string jsonString = File.ReadAllText(filename);
                Debug.WriteLine(filename);
                
                TrackerLight? trackLight = JsonSerializer.Deserialize<TrackerLight>(jsonString);
                if (trackLight?.location_solved != null)
                {
                    points.Add(new GeoPoint(
                        trackLight.location_solved.location.longitude,
                        trackLight.location_solved.location.latitude
                        ));
                    
                } 
                else
                {
                    TrackerFull? trackFull = JsonSerializer.Deserialize<TrackerFull>(jsonString);
                    if (trackFull.uplink_message.locations.frmpayload is not null)
                    {
                        points.Add(new GeoPoint(
                            trackFull.uplink_message.locations.frmpayload.longitude,
                            trackFull.uplink_message.locations.frmpayload.latitude
                            ));
                    }
                    else
                    {
                        points.Add(new GeoPoint(
                            trackFull.uplink_message.decoded_payload.longitude,
                            trackFull.uplink_message.decoded_payload.latitude
                            ));
                    }
                }
            }



            mapControl.Size = this.ClientSize;
            mapControl.Location = new Point(0, 0);
            mapControl.CacheFolder = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "MapControl");
            mapControl.TileServer = new OpenTopoMapServer();
            mapControl.MouseMove += (s, e) => UpdateWindowTitle();
            mapControl.MouseWheel += (s, e) => UpdateWindowTitle();
            mapControl.GotFocus += (s, e) =>
            {
                mapControl.Center = new GeoPoint(6.931933f, 46.992979f);
                mapControl.ZoomLevel = 13;
            };

            ClientSizeChanged += (s, e) =>
            {
                mapControl.Size = this.ClientSize;
                mapControl.Location = new Point(0, 0);
            };

            TrackStyle style = new TrackStyle(new Pen(Color.Red, 5f) { DashStyle = System.Drawing.Drawing2D.DashStyle.Dash });
            Track bike = new Track(style);
            bike.AddRange(points);
            mapControl.Tracks.Add(bike);
			mapControl.Focus();
        }

        private void UpdateWindowTitle()
        {
            GeoPoint g = mapControl.Mouse;
            this.Text = $"Tracker: Mouse = {g} / Zoom = {mapControl.ZoomLevel} / Bounding Box = TL:{mapControl.TopLeft}, TR:{mapControl.TopRight}, BR:{mapControl.BottomRight}, BL:{mapControl.BottomLeft}";
        }
    }
}